from .decorators.controller import api, controller, ctrl, router, rtr
from .decorators.handler import delete, get, head, options, patch, post, put, trace
from .decorators.module import dom, domain, mod, module
from .factory import Pest
from .metadata.types.injectable_meta import (
    ClassProvider,
    ExistingProvider,
    FactoryProvider,
    ProviderBase,
    Scope,
    SingletonProvider,
    ValueProvider,
)
from .utils.decorators import meta

__all__ = [
    'Pest',
    # decorators - module
    'module',
    'mod',
    'domain',
    'dom',
    # decorators - handler
    'get',
    'post',
    'put',
    'delete',
    'patch',
    'options',
    'head',
    'trace',
    # decorators - controller
    'controller',
    'ctrl',
    'router',
    'rtr',
    'api',
    # decorators - utils
    'meta',
    # meta - providers
    'ProviderBase',
    'ClassProvider',
    'ValueProvider',
    'SingletonProvider',
    'FactoryProvider',
    'ExistingProvider',
    'Scope',
]



from dataclasses import asdict
from typing import Any, Callable, Mapping, Type, TypeVar, Union

from ..metadata.meta import inject_metadata
from .protocols import DataclassInstance

Cls = TypeVar('Cls', bound=Type[Any])


def meta(meta: Union[Mapping[str, Any], DataclassInstance]) -> Callable[[Cls], Cls]:
    """🐀 » injects metadata into a class"""

    def wrapper(cls: Cls) -> Cls:
        if isinstance(meta, Mapping):
            metadata = meta
        else:
            metadata = asdict(meta)

        inject_metadata(cls, metadata)
        return cls

    return wrapper
